/*
**  Copyright (C) 1993-1996, OFFIS, Oldenburg University and CERIUM
**  
**  This software and supporting documentation were
**  developed by 
**  
**    Institut OFFIS
**    Bereich Kommunikationssysteme
**    Westerstr. 10-12
**    26121 Oldenburg, Germany
**    
**    Fachbereich Informatik
**    Abteilung Prozessinformatik
**    Carl von Ossietzky Universitaet Oldenburg 
**    Ammerlaender Heerstr. 114-118
**    26111 Oldenburg, Germany
**    
**    CERIUM
**    Laboratoire SIM
**    Faculte de Medecine
**    2 Avenue du Pr. Leon Bernard
**    35043 Rennes Cedex, France
**  
**  for CEN/TC251/WG4 as a contribution to the Radiological 
**  Society of North America (RSNA) 1993 Digital Imaging and 
**  Communications in Medicine (DICOM) Demonstration.
**  
**  THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND NEITHER OFFIS,
**  OLDENBURG UNIVERSITY NOR CERIUM MAKE ANY WARRANTY REGARDING 
**  THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY OR 
**  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER 
**  DISEASES OR ITS CONFORMITY TO ANY SPECIFICATION.  THE 
**  ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF THE SOFTWARE   
**  IS WITH THE USER. 
**  
**  Copyright of the software and supporting documentation
**  is, unless otherwise stated, jointly owned by OFFIS,
**  Oldenburg University and CERIUM and free access is hereby
**  granted as a license to use this software, copy this
**  software and prepare derivative works based upon this
**  software. However, any distribution of this software
**  source code or supporting documentation or derivative
**  works (source code and supporting documentation) must
**  include the three paragraphs of this copyright notice. 
** 
*/

#include "osconfig.h"

#ifndef X_DISPLAY_MISSING

#include <stdio.h>
#include <string.h>
#include <ctype.h> /* for isspace(), toupper() */

#include "ctnddico.h"

#include "dctk.h"
#include "dcutils.h"

#define GETTRACE 1 /* boolean: output trace data to stderr ? */
#define GETERROR 1 /* boolean: disply errors in GetImageData ? */

void malloc_error()
{
    fprintf(stderr, "Not enough Memory for Pixel_data! Fatal Error !\n");
    fprintf(stderr, "Please check with ipcs if there is shared memory left.\n");
    exit(-1);
}

/*****************/

/* remove spaces from string and convert to uppercase */

void  RemoveSpace(char *s1)
{
  char *source=s1;
  char *target=s1;
  if (source)
  {
    while (*source != '\0')
    {
      if (!isspace(*source)) *target++ = toupper(*source);
      source++;
    }
    *target = '\0';
  }
}

/*****************/

void GetImageData(
  char *fname,
  Image_Modul *image1,
  int Rescaleflag, /* should I read MLUT information? */
  int Windowflag,  /* should I read VLUT information? */
  int MULTI,       /* am I in a multi...? environment? */
  int CalcStatus)
{

  char *c;
  if (GETTRACE) fprintf(stderr, "loading DICOM file %s\n",fname);

  /* make sure data dictionary is loaded */
  if (!dcmDataDict.isDictionaryLoaded())
  {
    if (GETERROR) fprintf(stderr, "error: no data dictionary loaded, check environment variable: %s\n",
      DCM_DICT_ENVIRONMENT_VARIABLE);
    return;
  }

  DcmFileStream myin(fname, DCM_ReadMode);
  if ( myin.GetError() != EC_Normal ) 
  {
    if (GETERROR) fprintf(stderr, "error: cannot open DICOM file: %s\n", fname);
    return;
  }

  DcmFileFormat dfile;
  dfile.transferInit();
  dfile.read(myin, EXS_Unknown, EGL_noChange);
  dfile.transferEnd();

  if (dfile.error() != EC_Normal)
  {
    if (GETERROR) fprintf(stderr, "error: %s: reading file: %s\n", 
      dcmErrorConditionToString(dfile.error()),fname);
    return;
  }

  DcmDataset *dset = dfile.getDataset();
  if (dset == NULL)
  {
    if (GETERROR) fprintf(stderr, "error: file: %s contains no DICOM dataset\n",fname);
    return;
  }

  if (getSingleValue(dset, DCM_SamplesPerPixel, image1->Samples_Per_Pixel))
  {
    if (GETERROR) fprintf(stderr, "error: DCM_SamplesPerPixel not found.\n");
    return;
  }
  if (getSingleValue(dset, DCM_Rows, image1->Rows))
  {
    if (GETERROR) fprintf(stderr, "error: DCM_Rows not found.\n");
    return;
  }
  if (getSingleValue(dset, DCM_Columns, image1->Columns))
  {
    if (GETERROR) fprintf(stderr, "error: DCM_Columns not found.\n");
    return;
  }
  if (getSingleValue(dset, DCM_BitsAllocated, image1->Bits_Allocated))
  {
    if (GETERROR) fprintf(stderr, "error: DCM_BitsAllocated not found.\n");
    return;
  }
  if (getSingleValue(dset, DCM_BitsStored, image1->Bits_Stored))
  {
    if (GETERROR) fprintf(stderr, "error: DCM_BitsStored not found.\n");
    return;
  }
  if (getSingleValue(dset, DCM_HighBit, image1->High_Bit))
  {
    if (GETERROR) fprintf(stderr, "error: DCM_HighBit not found.\n");
    return;
  }
  if (getSingleValue(dset, DCM_PixelRepresentation, image1->Pixel_Representation))
  {
    if (GETERROR) fprintf(stderr, "error: DCM_PixelRepresentation not found.\n");
    return;
  }

  if (getSingleValue(dset, DCM_SOPInstanceUID, c))
  {
    if (GETERROR) fprintf(stderr, "error: DCM_SOPInstanceUID not found.\n");
    return;
  } else {
    if ((c==NULL)||(strlen(c)==0)||(strlen(c)>=MAX_STRING_LENGTH)) strcpy(image1->SOP, " "); 
    else strcpy(image1->SOP,c);
  }

  if (getSingleValue(dset, DCM_PhotometricInterpretation, c))
  {
    if (GETERROR) fprintf(stderr, "error: DCM_PhotometricInterpretation not found.\n");
    return;
  } else {
    if ((c==NULL)||(strlen(c)==0)||(strlen(c)>=20)) strcpy(image1->Photometric_Interpretation, " "); 
    else strcpy(image1->Photometric_Interpretation,c);
  }

  /* copy optional values */

  if (getSingleValue(dset, DCM_PatientName, c))
  {
    strcpy(image1->Patname," ");
  } else {
    if ((c==NULL)||(strlen(c)==0)||(strlen(c)>=MAX_STRING_LENGTH)) strcpy(image1->Patname, " "); 
    else strcpy(image1->Patname,c);
  }

  if (getSingleValue(dset, DCM_StudyID, c))
  {
    strcpy(image1->Study_ID," ");
  } else {
    if ((c==NULL)||(strlen(c)==0)||(strlen(c)>=MAX_STRING_LENGTH)) strcpy(image1->Study_ID, " "); 
    else strcpy(image1->Study_ID,c);
  }

  if (getSingleValue(dset, DCM_Modality, c))
  {
    strcpy(image1->Modality," ");
  } else {
    if ((c==NULL)||(strlen(c)==0)||(strlen(c)>=MAX_STRING_LENGTH)) strcpy(image1->Modality, " "); 
    else strcpy(image1->Modality,c);
  }

  if (getSingleValue(dset, DCM_BitsStored, image1->Planar_Configuration)) image1->Planar_Configuration=0;

  image1->Window_Center=0.0;
  if (getSingleValue(dset, DCM_WindowCenter, c))
  {
    if (c) sscanf(c, "%e", &image1->Window_Center);
  }

  image1->Window_Center=0.0;
  if (Windowflag) if (!getSingleValue(dset, DCM_WindowCenter, c))
  {
    if (c) sscanf(c, "%e", &image1->Window_Center);
  }
  image1->Window_Width=0.0;
  if (Windowflag) if (!getSingleValue(dset, DCM_WindowWidth, c))
  {
    if (c) sscanf(c, "%e", &image1->Window_Width);
  }
  image1->Rescale_Intercept=0.0;
  if (Rescaleflag) if (!getSingleValue(dset, DCM_RescaleIntercept, c))
  {
    if (c) sscanf(c, "%e", &image1->Rescale_Intercept);
  }
  image1->Rescale_Slope=1.0;
  if (Rescaleflag) if (!getSingleValue(dset, DCM_RescaleSlope, c))
  {
    if (c) sscanf(c, "%e", &image1->Rescale_Slope);
  }

  DcmStack stack;
  DcmObject *searchedObj=NULL;
  Uint16 *pUSvalueField=NULL;
  Sint16 *pSSvalueField=NULL;
  void *pvalueField=NULL;

  stack.clear();
  if (dset->search(DCM_PixelData, stack, ESM_fromHere, FALSE) == EC_Normal)
  {
    searchedObj = stack.top();
    if ((searchedObj->getLength()>0)&&(searchedObj->ident() == EVR_OW))
    {
	image1->Pixel_Data_Length = searchedObj->getLength();
        if ((!MULTI)||(CalcStatus != INEXT))
        {
          if (image1->Pixel_Data) free(image1->Pixel_Data);
          if (NULL == (image1->Pixel_Data = (char *)malloc(image1->Pixel_Data_Length+2))) malloc_error();
          if (EC_Normal != ((DcmElement *)searchedObj)->getUint16Array(pUSvalueField))
          {
            if (GETERROR) fprintf(stderr, "error: cannot read DCM_PixelData value.\n");
          } else {
#ifdef HAVE_MEMCPY
            memcpy(image1->Pixel_Data, pUSvalueField, image1->Pixel_Data_Length);            
#else
            bcopy((char *)pUSvalueField, image1->Pixel_Data, image1->Pixel_Data_Length);
#endif
          }
        }
    } else {
      if (GETERROR) fprintf(stderr, "error: DCM_PixelData empty or wrong VR.\n");
    }
  } else {
    if (GETERROR) fprintf(stderr, "error: DCM_PixelData not found.\n");
  }

  if ((MULTI) && (CalcStatus == INEXT)) return; /* DONE! */

  /* read Red Lookup Table Descriptor if present */
  stack.clear();
  if (dset->search(DCM_RedPaletteColorLookupTableDescriptor, stack, ESM_fromHere, FALSE) == EC_Normal)
  {
    searchedObj = stack.top();
    if ((searchedObj->getLength()==3*sizeof(Uint16))&&((searchedObj->ident() == EVR_US)||(searchedObj->ident() == EVR_SS)))
    {
      if (searchedObj->ident() == EVR_US)
      {
          if (EC_Normal != ((DcmElement *)searchedObj)->getUint16Array(pUSvalueField))
          {
            if (GETERROR) fprintf(stderr, "error: cannot read DCM_RedPaletteColorLookupTableDescriptor value.\n");
          } else pvalueField = pUSvalueField;
      } else {
          if (EC_Normal != ((DcmElement *)searchedObj)->getSint16Array(pSSvalueField))
          {
            if (GETERROR) fprintf(stderr, "error: cannot read DCM_RedPaletteColorLookupTableDescriptor value.\n");
          } else pvalueField = pSSvalueField;
      }
#ifdef HAVE_MEMCPY
      memcpy(image1->Red_Palette_Descriptors, pvalueField, 3*sizeof(Uint16));            
#else
      bcopy((char *)pvalueField, (char *)(image1->Red_Palette_Descriptors), 3*sizeof(Uint16));
#endif
    } else {
      if (GETERROR) fprintf(stderr, "error: DCM_RedPaletteColorLookupTableDescriptor VM or VR wrong.\n");
    }
  }

  /* read Green Lookup Table Descriptor if present */
  stack.clear();
  if (dset->search(DCM_GreenPaletteColorLookupTableDescriptor, stack, ESM_fromHere, FALSE) == EC_Normal)
  {
    searchedObj = stack.top();
    if ((searchedObj->getLength()==3*sizeof(Uint16))&&((searchedObj->ident() == EVR_US)||(searchedObj->ident() == EVR_SS)))
    {
      if (searchedObj->ident() == EVR_US)
      {
          if (EC_Normal != ((DcmElement *)searchedObj)->getUint16Array(pUSvalueField))
          {
            if (GETERROR) fprintf(stderr, "error: cannot read DCM_GreenPaletteColorLookupTableDescriptor value.\n");
          } else pvalueField = pUSvalueField;
      } else {
          if (EC_Normal != ((DcmElement *)searchedObj)->getSint16Array(pSSvalueField))
          {
            if (GETERROR) fprintf(stderr, "error: cannot read DCM_GreenPaletteColorLookupTableDescriptor value.\n");
          } else pvalueField = pSSvalueField;
      }
#ifdef HAVE_MEMCPY
      memcpy(image1->Green_Palette_Descriptors, pvalueField, 3*sizeof(Uint16));            
#else
      bcopy((char *)pvalueField, (char *)(image1->Green_Palette_Descriptors), 3*sizeof(Uint16));
#endif
    } else {
      if (GETERROR) fprintf(stderr, "error: DCM_GreenPaletteColorLookupTableDescriptor VM or VR wrong.\n");
    }
  }

  /* read Blue Lookup Table Descriptor if present */
  stack.clear();
  if (dset->search(DCM_BluePaletteColorLookupTableDescriptor, stack, ESM_fromHere, FALSE) == EC_Normal)
  {
    searchedObj = stack.top();
    if ((searchedObj->getLength()==3*sizeof(Uint16))&&((searchedObj->ident() == EVR_US)||(searchedObj->ident() == EVR_SS)))
    {
      if (searchedObj->ident() == EVR_US)
      {
          if (EC_Normal != ((DcmElement *)searchedObj)->getUint16Array(pUSvalueField))
          {
            if (GETERROR) fprintf(stderr, "error: cannot read DCM_BluePaletteColorLookupTableDescriptor value.\n");
          } else pvalueField = pUSvalueField;
      } else {
          if (EC_Normal != ((DcmElement *)searchedObj)->getSint16Array(pSSvalueField))
          {
            if (GETERROR) fprintf(stderr, "error: cannot read DCM_BluePaletteColorLookupTableDescriptor value.\n");
          } else pvalueField = pSSvalueField;
      }
#ifdef HAVE_MEMCPY
      memcpy(image1->Blue_Palette_Descriptors, pvalueField, 3*sizeof(Uint16));            
#else
      bcopy((char *)pvalueField, (char *)(image1->Blue_Palette_Descriptors), 3*sizeof(Uint16));
#endif
    } else {
      if (GETERROR) fprintf(stderr, "error: DCM_BluePaletteColorLookupTableDescriptor VM or VR wrong.\n");
    }
  }

  /* read Red Lookup Table Data if present */
  stack.clear();
  if (dset->search(DCM_RedPaletteColorLookupTableData, stack, ESM_fromHere, FALSE) == EC_Normal)
  {
    searchedObj = stack.top();
    if ((searchedObj->getLength()>0)&&((searchedObj->ident() == EVR_US)||(searchedObj->ident() == EVR_SS)))
    {
      if (searchedObj->ident() == EVR_US)
      {
          if (EC_Normal != ((DcmElement *)searchedObj)->getUint16Array(pUSvalueField))
          {
            if (GETERROR) fprintf(stderr, "error: cannot read DCM_RedPaletteColorLookupTableData value.\n");
          } else pvalueField=pUSvalueField;
      } else {
          if (EC_Normal != ((DcmElement *)searchedObj)->getSint16Array(pSSvalueField))
          {
            if (GETERROR) fprintf(stderr, "error: cannot read DCM_RedPaletteColorLookupTableData value.\n");
          } else pvalueField=pSSvalueField;
      }
      if (image1->Red_Palette_Data) free(image1->Red_Palette_Data);
      if (NULL == (image1->Red_Palette_Data = (unsigned short *)malloc(searchedObj->getLength()+2))) malloc_error();
#ifdef HAVE_MEMCPY
      memcpy(image1->Red_Palette_Data, pvalueField, searchedObj->getLength());            
#else
      bcopy((char *)pvalueField, image1->Red_Palette_Data, searchedObj->getLength());
#endif
    } else {
      if (GETERROR) fprintf(stderr, "error: DCM_RedPaletteColorLookupTableData VM or VR wrong.\n");
    }
  }

  /* read Green Lookup Table Data if present */
  stack.clear();
  if (dset->search(DCM_GreenPaletteColorLookupTableData, stack, ESM_fromHere, FALSE) == EC_Normal)
  {
    searchedObj = stack.top();
    if ((searchedObj->getLength()>0)&&((searchedObj->ident() == EVR_US)||(searchedObj->ident() == EVR_SS)))
    {
      if (searchedObj->ident() == EVR_US)
      {
          if (EC_Normal != ((DcmElement *)searchedObj)->getUint16Array(pUSvalueField))
          {
            if (GETERROR) fprintf(stderr, "error: cannot read DCM_GreenPaletteColorLookupTableData value.\n");
          } else pvalueField=pUSvalueField;
      } else {
          if (EC_Normal != ((DcmElement *)searchedObj)->getSint16Array(pSSvalueField))
          {
            if (GETERROR) fprintf(stderr, "error: cannot read DCM_GreenPaletteColorLookupTableData value.\n");
          } else pvalueField=pSSvalueField;
      }
      if (image1->Green_Palette_Data) free(image1->Green_Palette_Data);
      if (NULL == (image1->Green_Palette_Data = (unsigned short *)malloc(searchedObj->getLength()+2))) malloc_error();
#ifdef HAVE_MEMCPY
      memcpy(image1->Green_Palette_Data, pvalueField, searchedObj->getLength());            
#else
      bcopy((char *)pvalueField, image1->Green_Palette_Data, searchedObj->getLength());
#endif
    } else {
      if (GETERROR) fprintf(stderr, "error: DCM_GreenPaletteColorLookupTableData VM or VR wrong.\n");
    }
  }

  /* read Blue Lookup Table Data if present */
  stack.clear();
  if (dset->search(DCM_BluePaletteColorLookupTableData, stack, ESM_fromHere, FALSE) == EC_Normal)
  {
    searchedObj = stack.top();
    if ((searchedObj->getLength()>0)&&((searchedObj->ident() == EVR_US)||(searchedObj->ident() == EVR_SS)))
    {
      if (searchedObj->ident() == EVR_US)
      {
          if (EC_Normal != ((DcmElement *)searchedObj)->getUint16Array(pUSvalueField))
          {
            if (GETERROR) fprintf(stderr, "error: cannot read DCM_BluePaletteColorLookupTableData value.\n");
          } else pvalueField=pUSvalueField;
      } else {
          if (EC_Normal != ((DcmElement *)searchedObj)->getSint16Array(pSSvalueField))
          {
            if (GETERROR) fprintf(stderr, "error: cannot read DCM_BluePaletteColorLookupTableData value.\n");
          } else pvalueField=pSSvalueField;
      }
      if (image1->Blue_Palette_Data) free(image1->Blue_Palette_Data);
      if (NULL == (image1->Blue_Palette_Data = (unsigned short *)malloc(searchedObj->getLength()+2))) malloc_error();
#ifdef HAVE_MEMCPY
      memcpy(image1->Blue_Palette_Data, pvalueField, searchedObj->getLength());            
#else
      bcopy((char *)pvalueField, image1->Blue_Palette_Data, searchedObj->getLength());
#endif
    } else {
      if (GETERROR) fprintf(stderr, "error: DCM_BluePaletteColorLookupTableData VM or VR wrong.\n");
    }
  }

}

#endif /* X_DISPLAY_MISSING */
